"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnonCredsRsVerifierService = void 0;
const core_1 = require("@aries-framework/core");
const anoncreds_shared_1 = require("@hyperledger/anoncreds-shared");
const AnonCredsRsError_1 = require("../errors/AnonCredsRsError");
let AnonCredsRsVerifierService = class AnonCredsRsVerifierService {
    async verifyProof(agentContext, options) {
        const { credentialDefinitions, proof, proofRequest, revocationRegistries, schemas } = options;
        try {
            const presentation = anoncreds_shared_1.Presentation.load(JSON.stringify(proof));
            const rsCredentialDefinitions = {};
            for (const credDefId in credentialDefinitions) {
                rsCredentialDefinitions[credDefId] = anoncreds_shared_1.CredentialDefinition.load(JSON.stringify(credentialDefinitions[credDefId]));
            }
            const rsSchemas = {};
            for (const schemaId in schemas) {
                rsSchemas[schemaId] = anoncreds_shared_1.Schema.load(JSON.stringify(schemas[schemaId]));
            }
            const revocationRegistryDefinitions = {};
            const lists = [];
            for (const revocationRegistryDefinitionId in revocationRegistries) {
                const { definition, revocationStatusLists } = options.revocationRegistries[revocationRegistryDefinitionId];
                revocationRegistryDefinitions[revocationRegistryDefinitionId] = anoncreds_shared_1.RevocationRegistryDefinition.load(JSON.stringify(definition));
                for (const timestamp in revocationStatusLists) {
                    lists.push(anoncreds_shared_1.RevocationStatusList.create({
                        issuanceByDefault: true,
                        revocationRegistryDefinition: revocationRegistryDefinitions[revocationRegistryDefinitionId],
                        revocationRegistryDefinitionId,
                        timestamp: Number(timestamp),
                    }));
                }
            }
            return presentation.verify({
                presentationRequest: anoncreds_shared_1.PresentationRequest.load(JSON.stringify(proofRequest)),
                credentialDefinitions: rsCredentialDefinitions,
                schemas: rsSchemas,
                revocationRegistryDefinitions,
                revocationStatusLists: lists,
            });
        }
        catch (error) {
            throw new AnonCredsRsError_1.AnonCredsRsError('Error verifying proof', { cause: error });
        }
    }
};
AnonCredsRsVerifierService = __decorate([
    (0, core_1.injectable)()
], AnonCredsRsVerifierService);
exports.AnonCredsRsVerifierService = AnonCredsRsVerifierService;
//# sourceMappingURL=AnonCredsRsVerifierService.js.map